// The following code is example code from 
// "ActionScript 3.0 Bible, 2nd Edition" by Roger Braunstein
// Find all of the source code on the companion website
// http://actionscriptbible.com/
//
// Copyright (c) 2010 by Wiley Publishing, Inc., Indianapolis, Indiana
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.

package {
  import com.actionscriptbible.Example;

  public class ch12ex7 extends Example {
    public function ch12ex7() {
      //CONSTRUCTING REGULAR EXPRESSIONS
      
      trace("---- normal characters");
      trace("are you thinking what I'm thinking?".match(/think/g));
      //think,think
      trace("rub a dub dub, three men in a tub".match(/ub/g));
      //ub,ub,ub,ub
      
      trace("---- escaped characters");
      trace("c:\\windows\\"); //c:\windows\
      var re:RegExp = /I have a \$5 bill \& his\/her ID card\./;
      
      trace("---- character classes");
      var phoneNumberPattern:RegExp = /\d\d\d-\d\d\d-\d\d\d\d/;
      trace("the cat sat on the mat".match(/[msc]at/g)); //cat,sat,mat
      trace("abcdefghijklmnopqrstuvwxyz".match(/[a-cmx-z]/g)); //a,b,c,m,x,y,z      
      trace("roger dodger".match(/[^oge\s]/g)); //r,r,d,d,r
      
      trace("---- quantifiers");
      trace(/\w+:\s*\$\d+/.test("soup:          $40")); //true
      var betterPhoneNumber:RegExp = /\(?\d{3}\)?-?\d{3}-?\d{4}/;
      trace(betterPhoneNumber.test("(703)555-1234")); //true
      trace(betterPhoneNumber.test("310-555-1515")); //true
      trace(betterPhoneNumber.test("7245559090")); //true
      trace("a thousand thousandss!".match(/thousands*/g)); //thousand,thousandss      
      
      trace("---- anchors and boundaries");
      var contactText:String = 
          "Call us at one of these numbers.\n" +
          "Los Angeles: 310-555-2910\n" + 
          "New York: 212-555-2499\n" + 
          "Boston: 617-555-7141";
      var officeAndPhone:RegExp = /^([\w\s]+): (\d{3}-\d{3}-\d{4})/gm;
      trace(contactText.match(officeAndPhone));

      var officeAndPhone:RegExp = /^([\w\s]+): (\d{3}-\d{3}-\d{4})$/;
      trace(officeAndPhone.test(">>>New York: 212-555-2499")); //false
      trace(officeAndPhone.test("New York: 212-555-2499 (fax)")); //false

      var usingBoundary:RegExp = /\bzarflax\b/i;
      var noBoundary:RegExp = /zarflax/i;

      var garbage:String = "asndzarflaxhtewio";
      trace(usingBoundary.test(garbage)); //false
      trace(noBoundary.test(garbage)); //true
      
      var dialogue:String = "Nice try, Zarflax!";
      trace(usingBoundary.test(dialogue)); //true
      trace(noBoundary.test(dialogue)); //true
      
      var fakingBoundary:RegExp = /\Wzarflax\W/i;
      trace(garbage.match(fakingBoundary)); //(null)
      trace(dialogue.match(fakingBoundary)); // Zarflax!

      trace("---- alternation");
      var friends:RegExp = /leigh|mariko|neal|oskar|paula/gi;
      
      trace("---- groups");
      var rhymes:RegExp = /\b(tr|r|sp|b)a(c|s)e\b/gi;
      var str:String = "trace() the race to the orbiting base in outerspace";
      trace(str.match(rhymes)); //trace,race,base

      trace("the cat goes mew meow mewmew meow!".match(/(meo?w\s*)+/));
      //mew meow mewmew meow,meow
    }
  }
}