// The following code is example code from 
// "ActionScript 3.0 Bible, 2nd Edition" by Roger Braunstein
// Find all of the source code on the companion website
// http://actionscriptbible.com/
//
// Copyright (c) 2010 by Wiley Publishing, Inc., Indianapolis, Indiana
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.

package {
  import com.actionscriptbible.Example;
  
  public class ch12ex9 extends Example {
    public function ch12ex9() {
      //CONSTRUCTING ADVANCED EXPRESSIONS
      greedyMatchingSnippet();
      lazyMatchingSnippet();
      backreferencesSnippet();
      noncapturingSnippet();
      lookaheadSnippet();
    }
    protected function greedyMatchingSnippet():void {
      trace("---- greedy matching");
      //match <, (the tag name), any other stuff till >, >, (the inner text),
      //then </, (a tag name), and >.
      var getTag:RegExp = /< (\w+) \s* [^>]* >  (.*)  <\/ (\w+) >/x;
      var parts:Array;
      
      var htmlFragment1:String = '<li>item 1</li>';
      parts = htmlFragment1.match(getTag);
      trace("nodeName:", parts[1]); //nodeName: li
      trace("innerHtml:", parts[2]); //innerHtml: item 1
      
      var htmlFragment2:String = '<li>item 1</li><li>item 2</li>';
      parts = htmlFragment2.match(getTag);
      trace("nodeName:", parts[1]); //nodeName: li
      trace("innerHtml:", parts[2]); //innerHtml: item1</li><li>item 2
    }
    protected function lazyMatchingSnippet():void {
      trace("---- lazy matching");
      //match <, (the tag name), any other stuff till >, >, (the inner text),
      //then </, (a tag name), and >.
      var getTag:RegExp = /< (\w+) \s* [^>]* >  (.*?)  <\/ (\w+) >/gx;
      var htmlFragment:String = '<li>item 1</li><li>item 2</li>';
      var parts:Array;
      while(parts = getTag.exec(htmlFragment))
      {
          trace("nodeName:", parts[1]);
          trace("innerHtml:", parts[2]);
      }
      //nodeName: li
      //innerHtml: item 1
      //nodeName: li
      //innerHtml: item 2
    }
    protected function backreferencesSnippet():void {
      trace("---- backreferences");
      var re:RegExp = /\b (\w{3}) \b  .*?  \b \1 \b /ix;
      var testString:String = "far, farther fetched free fun fetched fun free";
      trace(testString.match(re)); //fun fetched fun,fun
    }
    protected function noncapturingSnippet():void {
      trace("---- noncapturing groups");
      var re:RegExp = /(?:i|you|he|she|it|we|they) likes? to (\w+)/i;
      var match:Object = re.exec("We like to party.");
      trace(match[1]); //party
    }
    protected function lookaheadSnippet():void {
      trace("---- positive lookahead groups");
      var re:RegExp = /\b (?:\w{3}) \b  (?= .*?  \b \1 \b) \s* (\w+) /x;
      var testString:String = "far, farther fetched free fun fetched fun free";
      trace(testString.match(re)); //fun fetched,fetched
      
    }
  }
}