// The following code is example code from 
// "ActionScript 3.0 Bible, 2nd Edition" by Roger Braunstein
// Find all of the source code on the companion website
// http://actionscriptbible.com/
//
// Copyright (c) 2010 by Wiley Publishing, Inc., Indianapolis, Indiana
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.

package  {
  import com.actionscriptbible.Example;
  import flash.events.Event;

  public class ch20ex2 extends Example {
    public function ch20ex2() {
      var t:Thermometer = new Thermometer();
      t.addEventListener(Thermometer.TEMP_CHANGED, onTempChanged);
      
      var warn:TempWarning = new TempWarning(t);
      warn.x = stage.stageWidth - warn.width;
      addChild(warn);
      
      t.debugSimulateCrazyWeather();
    }
    
    protected function onTempChanged(event:Event):void {
      var t:Thermometer = Thermometer(event.target);
      trace("It's now " + t.temp.toFixed(1) + "°F");
    }
  }
}

import flash.display.Sprite;
import flash.geom.ColorTransform;
class TempWarning extends Sprite {
  public function TempWarning(t:Thermometer) {
    graphics.beginFill(0xffff00);
    graphics.drawRect(0, 0, 50, 50);
    graphics.endFill();
    t.addEventListener(Thermometer.TEMP_CHANGED, onTempChanged);
  }
  
  protected function onTempChanged(event:Event):void {
    var t:Thermometer = Thermometer(event.target);
    var ctx:ColorTransform = new ColorTransform();
    var hotness:Number = mapRange(t.temp, 60, 100);
    ctx.redMultiplier = hotness;
    ctx.greenMultiplier = 1 - hotness;
    this.transform.colorTransform = ctx;
  }
  
  //map a range of values from min->max to 0->1 (and clamp)
  protected function mapRange(value:Number, min:Number, max:Number):Number {
    return Math.min(1, Math.max(0, (value - min) / (max - min)));
  }
}

import flash.events.*;
import flash.utils.setInterval;
class Thermometer extends EventDispatcher {
  protected var _temp:Number = 72;
  public static const TEMP_CHANGED:String = "tempChanged";

  public function set temp(newTemp:Number):void {
    _temp = newTemp;
    dispatchEvent(new Event(TEMP_CHANGED));
  }
  
  public function get temp():Number {
    return _temp;
  }
  
  internal function debugSimulateCrazyWeather():void {
    //tweak the weather to trend continuously higher (to see our handiwork)
    setInterval(function():void {temp += 3 * (Math.random() - 0.4);}, 100);
  }
}