// The following code is example code from 
// "ActionScript 3.0 Bible, 2nd Edition" by Roger Braunstein
// Find all of the source code on the companion website
// http://actionscriptbible.com/
//
// Copyright (c) 2010 by Wiley Publishing, Inc., Indianapolis, Indiana
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.

package {
  import flash.display.*;
  import flash.events.*;
  import flash.geom.*;
  import flash.net.*;
  public class ch38ex3 extends Sprite {
    protected const URL_BASE:String = "http://actionscriptbible.com/files/";
    protected const INITIAL_PARTICLES:int = 1000;
    protected const WELL_GRAVITY_MAX:Number = 5000;
    protected var W:int, H:int; 

    protected var generatorShader:Shader;
    protected var stepShader:Shader;
    protected var forcemap:BitmapData;
    protected var particleData:Vector.<Number>; //4 channels: px,py,vx,vy
    
    protected var bmp:BitmapData;
    protected var bitmap:Bitmap;
    protected const DIM:ColorTransform = new ColorTransform(.9, .9, .9, .95);
    public function ch38ex3() {
      stage.frameRate = 60;
      loadShader("GravitySimGenerateMap.pbj", onGeneratorShaderReady);
    }
    protected function loadShader(url:String, onComplete:Function):void {
      var loader:URLLoader = new URLLoader(new URLRequest(URL_BASE + url));
      loader.dataFormat = URLLoaderDataFormat.BINARY;
      loader.addEventListener(Event.COMPLETE, onComplete);
    }
    protected function onGeneratorShaderReady(event:Event):void {
      event.target.removeEventListener(Event.COMPLETE, onGeneratorShaderReady);
      generatorShader = new Shader(URLLoader(event.target).data);
      
      forcemap = new BitmapData(stage.stageWidth, stage.stageHeight, false, 0);
      W = forcemap.width; H = forcemap.height;
      var forceBitmap:Bitmap = new Bitmap(forcemap);
      addChild(forceBitmap);
      bmp = new BitmapData(W, H, false, 0);
      bitmap = new Bitmap(bmp);
      addChild(bitmap);
      bitmap.alpha = .99;
      
      initializeParticles();
      initializeWells();
      loadShader("GravitySimStep.pbj", onStepReady);
    }
    protected function onStepReady(event:Event):void {
      event.target.removeEventListener(Event.COMPLETE, onGeneratorShaderReady);
      stepShader = new Shader(URLLoader(event.target).data);
      
      stage.addEventListener(MouseEvent.CLICK,
        function(event:Event):void{initializeParticles()});
      stage.addEventListener(KeyboardEvent.KEY_UP,
        function(event:Event):void{initializeWells()});
      addEventListener(Event.ENTER_FRAME, onEnterFrame);
    }
    protected function initializeParticles():void {
      var r:Function = Math.random;
      particleData = new Vector.<Number>();
      for (var i:int = 0; i < INITIAL_PARTICLES; i++) {
        particleData.push(r()*W, r()*H, (r()-0.5)*4, (r()-0.5)*4);
      }
    }
    protected function initializeWells():void {
      var r:Function = Math.random;
      var data:ShaderData = generatorShader.data;
      data.power.value = [1.4];
      data.star0.value = [r()*W/2 + W/4, r()*H/2 + H/4, r()*WELL_GRAVITY_MAX];
      data.star1.value = [r()*W/2 + W/4, r()*H/2 + H/4, r()*WELL_GRAVITY_MAX];
      var shaderJob:ShaderJob = new ShaderJob(generatorShader, forcemap);
      shaderJob.start(true);
    }
    protected function step():void {
      var data:ShaderData = stepShader.data;
      data.forcemap.input = forcemap;
      data.inpoints.input = particleData;
      data.inpoints.height = 1;
      data.inpoints.width = particleData.length/4;
      var shaderJob:ShaderJob = new ShaderJob(stepShader, particleData, 
        particleData.length/4, 1);
      shaderJob.start(true);
    } 
    protected function draw():void {
      bmp.colorTransform(bmp.rect, DIM);
      bmp.lock();
      for (var i:int = 0; i < particleData.length; i+=4) {
        bmp.setPixel(particleData[i], particleData[i+1], 0xffffff);
      }
      bmp.unlock();
    }
    protected function onEnterFrame(event:Event):void {
      step();
      draw();
    }
  }
}